{-# LANGUAGE UndecidableInstances, FlexibleInstances, MultiParamTypeClasses  #-}

{-| Module  : FiniteCategories
Description : Full subcategory a category C.
Copyright   : Guillaume Sabbagh 2021
License     : GPL-3
Maintainer  : guillaumesabbagh@protonmail.com
Stability   : experimental
Portability : portable

Full subcategory a category C.
-}
module Subcategories.FullSubcategory
(
FullSubcategory(..)
)
where
    import FiniteCategory.FiniteCategory
    import Data.List (nub)
    import Utils.SetList
    import IO.PrettyPrint
    
    -- | The datatype for full subcategories of a given category containing given objects.
    data FullSubcategory c m o = FullSubcategory c [o] deriving (Eq, Show)
    
    instance (FiniteCategory c m o, Morphism m o, Eq m, Eq o) => FiniteCategory (FullSubcategory c m o) m o where
        ob (FullSubcategory _ objs) = nub $ objs
        identity (FullSubcategory c objs) obj = if elem obj objs then identity c obj else error "Cannot create identity of an object not in the category."
        ar (FullSubcategory c objs) s t = if elem s objs && elem t objs then ar c s t else error "Cannot create morphisms between objects not in the category."
        
    instance (FiniteCategory c m o, Morphism m o, Eq m, Eq o) => GeneratedFiniteCategory (FullSubcategory c m o) m o where
        genAr = defaultGenAr
        decompose = defaultDecompose
        
    instance (PrettyPrintable c, PrettyPrintable o) => PrettyPrintable (FullSubcategory c m o) where
        pprint (FullSubcategory c o) = "Full subcategory of "++pprint c++" containing "++pprint o